<?php
namespace Bitrix\Main\IO;

class Directory
	extends DirectoryEntry
{
	public function __construct($path)
	{
		parent::__construct($path);
	}

	public function IsExists()
	{
		$p = $this->GetPhysicalPath();
		return file_exists($p) && is_dir($p);
	}

	public function Delete()
	{
		return self::DeleteInternal($this->GetPhysicalPath());
	}

	private static function DeleteInternal($path)
	{
		if (is_file($path) || is_link($path))
		{
			if (!@unlink($path))
				throw new FileDeleteException($path);
		}
		elseif (is_dir($path))
		{
			if ($handle = opendir($path))
			{
				while (($file = readdir($handle)) !== false)
				{
					if ($file == "." || $file == "..")
						continue;

					self::DeleteInternal(Path::Combine($path, $file));
				}
				closedir($handle);
			}
			if (!@rmdir($path))
				throw new FileDeleteException($path);
		}

		return true;
	}

	/**
	 * @return array|FileSystemEntry[]
	 * @throws FileNotFoundException
	 */
	public function GetChildren()
	{
		if (!$this->IsExists())
			throw new FileNotFoundException($this->originalPath);

		$arResult = array();

		if ($handle = opendir($this->GetPhysicalPath()))
		{
			while (($file = readdir($handle)) !== false)
			{
				if ($file == "." || $file == "..")
					continue;

				$pathLogical = Path::Combine($this->path, Path::ConvertPhysicalToLogical($file));
				$pathPhysical = Path::Combine($this->GetPhysicalPath(), $file);
				if (is_dir($pathPhysical))
					$arResult[] = new Directory($pathLogical);
				else
					$arResult[] = new File($pathLogical);
			}
			closedir($handle);
		}

		return $arResult;
	}

	/**
	 * @param $name
	 * @return Directory|DirectoryEntry
	 */
	public function CreateSubdirectory($name)
	{
		$dir = new Directory(Path::Combine($this->path, $name));
		if (!$dir->IsExists())
			mkdir($dir->GetPhysicalPath(), BX_DIR_PERMISSIONS, true);
		return $dir;
	}

	public function GetCreationTime()
	{
		if (!$this->IsExists())
			throw new FileNotFoundException($this->originalPath);

		return filectime($this->GetPhysicalPath());
	}

	public function GetLastAccessTime()
	{
		if (!$this->IsExists())
			throw new FileNotFoundException($this->originalPath);

		return fileatime($this->GetPhysicalPath());
	}

	public function GetModificationTime()
	{
		if (!$this->IsExists())
			throw new FileNotFoundException($this->originalPath);

		return filemtime($this->GetPhysicalPath());
	}

	public function MarkWritable()
	{
		if (!$this->IsExists())
			throw new FileNotFoundException($this->originalPath);

		@chmod($this->GetPhysicalPath(), BX_DIR_PERMISSIONS);
	}

	public function GetPermissions()
	{
		return fileperms($this->GetPhysicalPath());
	}

	public static function CreateDirectory($path)
	{
		$dir = new self($path);
		$dir->Create();
	}

	public static function DeleteDirectory($path)
	{
		$dir = new self($path);
		$dir->Delete();
	}
}
