<?php
namespace Bitrix\Main\Localization;

use \Bitrix\Main\Application;
use \Bitrix\Main\Entity;

class LanguageEntity extends Entity\Base
{
	public function initialize()
	{
		$this->className = __CLASS__;
		$this->filePath = __FILE__;

		$this->dbTableName = 'b_language';

		$this->fieldsMap = array(
			'LID' => array(
				'data_type' => 'string',
				'primary' => true,
			),
			'SORT' => array(
				'data_type' => 'integer',
			),
			'DEF' => array(
				'data_type' => 'boolean',
				'values' => array('N', 'Y'),
			),
			'ACTIVE' => array(
				'data_type' => 'boolean',
				'values' => array('N', 'Y'),
			),
			'NAME' => array(
				'data_type' => 'string',
			),
			'CULTURE_ID' => array(
				'data_type' => 'integer',
			),
			'CULTURE' => array(
				'data_type' => 'Bitrix\Main\Localization\Culture',
				'reference' => array('=this.CULTURE_ID' => 'ref.ID'),
			),
		);
	}
}

class Language extends Entity\DataManager
{
	/**
	 * @param \Bitrix\Main\Application $application
	 * @return array|bool
	 */
	public static function getCurrent(Application $application)
	{
		$context = $application->getContext();
		$request = $context->getRequest();

		$defaultLang = $request["lang"];
		if($defaultLang == '')
			$defaultLang = \COption::GetOptionString("main", "admin_lid");

		if($defaultLang <> '')
		{
			$langDb = Language::getById($defaultLang);
			if(($arLanguage = $langDb->Fetch()))
				return $arLanguage;
		}

		return false;
	}
}
