<?php
namespace Bitrix\Main;

use \Bitrix\Main\Text;
use \Bitrix\Main\IO;

abstract class Request
	implements \ArrayAccess, \Iterator, \Countable
{
	/**
	 * @var Context
	 */
	protected $context;

	protected $request = array();
	protected $requestedFile = "";
	protected $requestedFileDirectory = "";

	protected $isAdminSection = null;

	public function __construct(Context $context, array $request)
	{
		$this->context = $context;
		$this->request = $request;
	}

	public function getRequest($name)
	{
		if (isset($this->request[$name]))
			return $this->request[$name];
		return null;
	}

	public function offsetExists($name)
	{
		return array_key_exists($name, $this->request);
	}

	public function offsetGet($name)
	{
		if (isset($this->request[$name]))
			return $this->request[$name];
		return false;
	}

	public function offsetSet($name, $value)
	{
		throw new NotSupportedException("Can not set request");
	}

	public function offsetUnset($name)
	{
		throw new NotSupportedException("Can not unset request");
	}

	public function current()
	{
		return current($this->request);
	}

	public function next()
	{
		return next($this->request);
	}

	public function key()
	{
		return key($this->request);
	}

	public function valid()
	{
		$key = $this->key();
		return isset($this->request[$key]);
	}

	public function rewind()
	{
		return reset($this->request);
	}

	public function count()
	{
		return count($this->request);
	}

	public function getPhpSelf()
	{
		$server = $this->context->GetServer();
		return $server->getPhpSelf();
	}

	public function getScriptName()
	{
		$server = $this->context->GetServer();
		return $server->getScriptName();
	}

	public function getRequestedPage()
	{
		if ($this->requestedFile !== "")
			return $this->requestedFile;

		$page = $this->getScriptName();

		$page = IO\Path::Normalize($page);

		if (IO\Path::Validate($page))
			return $this->requestedFile = $page;

		throw new SystemException("Script name is not valid");
	}

	public function getRequestedPageDirectory()
	{
		if ($this->requestedFileDirectory !== "")
			return $this->requestedFileDirectory;

		if ($this->requestedFile === "")
			$this->getRequestedPage();

		return $this->requestedFileDirectory = IO\Path::GetDirectory($this->requestedFile);
	}
}
